import * as fs from 'fs';
import ytdl from 'ytdl-core';
export class YoutubeDownloader {
    async download(url) {
        return new Promise(async (resolve, reject) => {
            try {
                this.validateUrl(url);
                const info = await ytdl.getInfo(url);
                const title = info.player_response.videoDetails.title;
                const stream = ytdl.downloadFromInfo(info);
                stream.pipe(fs.createWriteStream(`${this.fileName(title)}.mp4`));
                resolve({
                    message: `Downloading '${title}'...`,
                    stream: stream,
                });
            }
            catch (error) {
                reject(error);
            }
        });
    }
    validateUrl(url) {
        if (!url || !ytdl.validateURL(url)) {
            throw new Error('Plese enter a valid YouTube URL.');
        }
    }
    fileName(title) {
        return title.toLowerCase()
            .replaceAll(/[^\dA-Za-z\s]/g, '')
            .replaceAll(/\s+/g, ' ')
            .replaceAll(' ', '-');
    }
}
